#include <stdlib.h>
#include <ggi/libggi.h>
#include "handy.h"
#include "acidwarp.h"
#include "palinit.h"

/* Initialzes a palette array to one of the palette types      */
void initPalArray (UINT *palArray, int pal_type)
{
	switch (pal_type)
	{
		case RGBW_PAL:
			init_rgbw_palArray (palArray);
		break;

		case W_PAL:
			init_w_palArray (palArray);
		break;

		case W_HALF_PAL:
			init_w_half_palArray (palArray);
		break;

		case PASTEL_PAL:
			init_pastel_palArray (palArray);
		break;

		case RGBW_LIGHTNING_PAL:
		      init_rgbw_palArray (palArray);
		      add_sparkles_to_palette (palArray, 9);
                break;

		case W_LIGHTNING_PAL:
		      init_w_palArray (palArray);
		      add_sparkles_to_palette (palArray, 9);
	        break;

		case W_HALF_LIGHTNING_PAL:
                      init_w_half_palArray (palArray);
                      add_sparkles_to_palette (palArray, 9);
                break;

		case PASTEL_LIGHTNING_PAL:
                      init_pastel_palArray (palArray);
                      add_sparkles_to_palette (palArray, 9);
                break;
		default:
			init_w_palArray(palArray);
		break;
	}
}


void add_sparkles_to_palette (UINT *palArray, int sparkle_amount)
{
  int palRegNum;

  for (palRegNum = 1; palRegNum < 256;  palRegNum += 4)
  {
    palArray[palRegNum * 3    ] = (MIN(63 * 1024, palArray[palRegNum * 3    ] + sparkle_amount * 1024));
    palArray[palRegNum * 3 + 1] = (MIN(63 * 1024, palArray[palRegNum * 3 + 1] + sparkle_amount * 1024));
    palArray[palRegNum * 3 + 2] = (MIN(63 * 1024, palArray[palRegNum * 3 + 2] + sparkle_amount * 1024));
  }
}

void init_rgbw_palArray (UINT *palArray)
{
	int palRegNum;

	for (palRegNum = 0; palRegNum < 32; ++palRegNum)
	{
		palArray[ palRegNum        * 3    ] = (UINT)palRegNum * 8 * 256;
		palArray[(palRegNum +  64) * 3    ] = (UINT)0;
		palArray[(palRegNum + 128) * 3    ] = (UINT)0;
		palArray[(palRegNum + 192) * 3    ] = (UINT)palRegNum * 8 * 256;

		palArray[ palRegNum        * 3 + 1] = (UINT)0;
		palArray[(palRegNum +  64) * 3 + 1] = (UINT)palRegNum * 8 * 256;
		palArray[(palRegNum + 128) * 3 + 1] = (UINT)0;
		palArray[(palRegNum + 192) * 3 + 1] = (UINT)palRegNum * 8 * 256;

		palArray[ palRegNum        * 3 + 2] = (UINT)0;
		palArray[(palRegNum +  64) * 3 + 2] = (UINT)0;
		palArray[(palRegNum + 128) * 3 + 2] = (UINT)palRegNum * 8 * 256;
		palArray[(palRegNum + 192) * 3 + 2] = (UINT)palRegNum * 8 * 256;
	}

	for (palRegNum = 32; palRegNum < 64; ++palRegNum)
	{
		palArray[ palRegNum        * 3    ] = (UINT)(63 - palRegNum) * 8 * 256;
		palArray[(palRegNum +  64) * 3    ] = (UINT)0;
		palArray[(palRegNum + 128) * 3    ] = (UINT)0;
		palArray[(palRegNum + 192) * 3    ] = (UINT)(63 - palRegNum) * 8 * 256;

		palArray[ palRegNum        * 3 + 1] = (UINT)0;
		palArray[(palRegNum +  64) * 3 + 1] = (UINT)(63 - palRegNum) * 8 * 256;
		palArray[(palRegNum + 128) * 3 + 1] = (UINT)0;
		palArray[(palRegNum + 192) * 3 + 1] = (UINT)(63 - palRegNum) * 8 * 256;

		palArray[ palRegNum        * 3 + 2] = (UINT)0;
		palArray[(palRegNum +  64) * 3 + 2] = (UINT)0;
		palArray[(palRegNum + 128) * 3 + 2] = (UINT)(63 - palRegNum) * 8 * 256;
		palArray[(palRegNum + 192) * 3 + 2] = (UINT)(63 - palRegNum) * 8 * 256;
	}
}


void init_w_palArray (UINT *palArray)
{
        int palRegNum;

        for (palRegNum = 0; palRegNum < 128; ++palRegNum)
        {
                palArray[palRegNum * 3    ] = (UINT)palRegNum * 512;
                palArray[palRegNum * 3 + 1] = (UINT)palRegNum * 512;
                palArray[palRegNum * 3 + 2] = (UINT)palRegNum * 512;
        }

        for (palRegNum = 128; palRegNum < 256; ++palRegNum)
        {
                palArray[palRegNum * 3    ] = (UINT)(255 - palRegNum) * 512;
                palArray[palRegNum * 3 + 1] = (UINT)(255 - palRegNum) * 512;
                palArray[palRegNum * 3 + 2] = (UINT)(255 - palRegNum) * 512;
        }

}

void init_w_half_palArray (UINT *palArray)
{
	int palRegNum;

	for (palRegNum = 0; palRegNum < 64; ++palRegNum)
	{
		palArray[palRegNum        * 3    ] = (UINT)palRegNum * 1024;
		palArray[palRegNum        * 3 + 1] = (UINT)palRegNum * 1024;
		palArray[palRegNum        * 3 + 2] = (UINT)palRegNum * 1024;

		palArray[(palRegNum + 64) * 3    ] = (UINT)(63 - palRegNum) * 1024;
		palArray[(palRegNum + 64) * 3 + 1] = (UINT)(63 - palRegNum) * 1024;
		palArray[(palRegNum + 64) * 3 + 2] = (UINT)(63 - palRegNum) * 1024;
	}

	for (palRegNum = 128; palRegNum < 256; ++palRegNum)
	{
		palArray[palRegNum        * 3    ] = 0;
		palArray[palRegNum        * 3 + 1] = 0;
		palArray[palRegNum        * 3 + 2] = 0;
	}
}

void init_pastel_palArray (UINT *palArray)
{
	int palRegNum;

	for (palRegNum = 0; palRegNum < 128; ++palRegNum)
	{
		palArray[ palRegNum        * 3    ] = (UINT)((31 + palRegNum/4)*1024);
		palArray[ palRegNum        * 3 + 1] = (UINT)((31 + palRegNum/4)*1024);
		palArray[ palRegNum        * 3 + 2] = (UINT)((31 + palRegNum/4)*1024);

		palArray[(palRegNum + 128) * 3    ] = (UINT)((31 + (127 - palRegNum)/4) * 1024);
		palArray[(palRegNum + 128) * 3 + 1] = (UINT)((31 + (127 - palRegNum)/4) * 1024);
		palArray[(palRegNum + 128) * 3 + 2] = (UINT)((31 + (127 - palRegNum)/4) * 1024);
	}
}
